<?php

/**
 * This software is intended for use with Oxwall Free Community Software http://www.oxwall.org/ and is
 * licensed under The BSD license.

 * ---
 * Copyright (c) 2009, Skalfa LLC
 * All rights reserved.

 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:
 *
 *  - Redistributions of source code must retain the above copyright notice, this list of conditions and
 *  the following disclaimer.
 *
 *  - Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *  the following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *  - Neither the name of the Skalfa LLC nor the names of its contributors may be used to endorse or promote products
 *  derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * Image Service Class.  
 * 
 * @author Egor Bulgakov <plugins@oxwallcs.com>
 * @package ow.plugin.ocsslider.bol
 * @since 1.0
 */
final class OCSSLIDER_BOL_ImageService
{
    /**
     * @var OCSSLIDER_BOL_ImageDao
     */
    private $imageDao;
    /**
     * Class instance
     *
     * @var OCSSLIDER_BOL_ImageService
     */
    private static $classInstance;

    const IMAGE_PREFIX = 'image_';
    
    /**
     * Class constructor
     *
     */
    private function __construct()
    {
        $this->imageDao = OCSSLIDER_BOL_ImageDao::getInstance();
    }

    /**
     * Returns class instance
     *
     * @return OCSSLIDER_BOL_ImageService
     */
    public static function getInstance()
    {
        if ( null === self::$classInstance )
        {
            self::$classInstance = new self();
        }

        return self::$classInstance;
    }

    /**
     * Adds image
     *
     * @param OCSSLIDER_BOL_Image $image
     * @return int
     */
    public function addImage( OCSSLIDER_BOL_Image $image, $file )
    {
        $this->imageDao->save($image);

        $path = $this->getImagePath($image->id, $image->hash, $image->ext);
        
        if ( !move_uploaded_file($file['tmp_name'], $path) )
        {
        	$this->imageDao->deleteById($image->id);
        	
        	return false;
        }
        
        return $image->id;
    }
    
    public function updateImageDto( OCSSLIDER_BOL_Image $image )
    {
    	$this->imageDao->save($image);
    	return true;
    } 
    
    public function updateImageFile( OCSSLIDER_BOL_Image $image, $file )
    {
    	$image->hash = time();

        $path = $this->getImagePath($image->id, $image->hash, $image->ext);
        
        if ( move_uploaded_file($file, $path) )
        {
            $this->imageDao->save($image);
            return true;
        }
        
        return false;
    }
    
    public function deleteImage( $imageId )
    {
    	if ( !$imageId )
    	{
    		return false;
    	}
    	
    	$image = $this->imageDao->findById($imageId);
    	
    	if ( !$image )
    	{
    		return false;
    	}
    	
    	if ( $this->imageDao->deleteById($imageId) )
    	{
    	   $path = $this->getImagePath($imageId, $image->hash, $image->ext);
           @unlink($path);
            
            return true;
        }
    	
    	return false;
    }
    
    public function findById( $id )
    {
    	if ( !$id )
    	{
    		return false;
    	}
    	
    	return $this->imageDao->findById($id);
    }
    
    public function getImagePath( $imageId, $hash, $ext )
    {        
        $userfilesDir = OW::getPluginManager()->getPlugin('ocsslider')->getUserFilesDir();
        
        return $userfilesDir . self::IMAGE_PREFIX . $imageId . '_' . $hash . '.' . $ext;
    }

    public function getImageUrl( $imageId, $hash, $ext )
    {        
        $userfilesUrl = OW::getPluginManager()->getPlugin('ocsslider')->getUserFilesUrl();
        
        return $userfilesUrl . self::IMAGE_PREFIX . $imageId . '_' . $hash . '.' . $ext;
    }
    
    public function getNextOrder()
    {
        return $this->imageDao->getMaxOrder() + 1;
    }
    
    public function getImageList( )
    {
    	$list = $this->imageDao->getImageList();
    	
    	$return = array();
    	foreach ( $list as $image )
    	{
    		$return[$image->id]['dto'] = $image;
    		$return[$image->id]['imageUrl'] = $this->getImageUrl($image->id, $image->hash, $image->ext);
    	}
    	
    	return $return;
    }
    
    public function changeImageOrder( $imageId, $move = 'up' )
    {
    	$image = $this->imageDao->findById($imageId);
    	if ( !$image )
    	{
    		return false;
    	}
    	
    	$currentOrder = $image->order;
    	 
    	switch ( $move )
    	{
    		case 'up':
    			$destImage = $this->imageDao->findPreviousImage($imageId, $currentOrder);
    			break;
    			
    		case 'down':
    			$destImage = $this->imageDao->findNextImage($imageId, $currentOrder);
    			break;
    	}
    	
    	if ( !$destImage )
        {
            return false;
        }
                
        $image->order = $destImage->order;
        $this->imageDao->save($image);
                
        $destImage->order = $currentOrder;
        $this->imageDao->save($destImage);
        
        return true;
    }
}