<?php

/**
 * This software is intended for use with Oxwall Free Community Software http://www.oxwall.org/ and is
 * licensed under The BSD license.

 * ---
 * Copyright (c) 2009, Skalfa LLC
 * All rights reserved.

 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:
 *
 *  - Redistributions of source code must retain the above copyright notice, this list of conditions and
 *  the following disclaimer.
 *
 *  - Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *  the following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *  - Neither the name of the Skalfa LLC nor the names of its contributors may be used to endorse or promote products
 *  derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * @author Egor Bulgakov <plugins@oxwallcs.com>
 * @package ow_plugins.ocsslider.components
 * @since 1.0
 */
class OCSSLIDER_CMP_SliderWidget extends BASE_CLASS_Widget
{
    public function __construct( BASE_CLASS_WidgetParameter $params )
    {
        parent::__construct();

        $staticUrl = OW::getPluginManager()->getPlugin('ocsslider')->getStaticUrl();
        $this->assign('url', $staticUrl . 'slides.1.0.9/img/');

        OW::getDocument()->addScript($staticUrl . 'slides.1.0.9/js/slides.min.jquery.js');
        OW::getDocument()->addStyleSheet($staticUrl.'slides.1.0.9/css/global.css');
        
        $imageService = OCSSLIDER_BOL_ImageService::getInstance();
        
        $list = $imageService->getImageList();
        
        if ( !count($list) )
        {
        	$this->setVisible(false);
        }
               
        $this->assign('images', $list);
        
        $configs = json_decode(OW::getConfig()->getValue('ocsslider', 'configuration'), true);

        OW::getDocument()->addOnloadScript("
	        $(function(){
	            $('#slides').slides({
	                preload: " . ($configs['preload'] ? "true" : "false") . ",
	                preloadImage: '" . $staticUrl . "slides.1.0.9/img/loading.gif',
	                crossfade: " . ($configs['crossfade'] ? "true" : "false") . ",
	                randomize: " . ($configs['randomize'] ? "true" : "false") . ",
	                generatePagination: " . ($configs['pagination'] ? "true" : "false") . ",
	                fadeSpeed : " . $configs['speed'] . ",
	                slideSpeed : " . $configs['speed'] . ",
	                play: " . $configs['play'] . ",
	                pause: 2500,
	                hoverPause: " . ($configs['hoverPause'] ? "true" : "false") . ",
	                effect: '" . $configs['effect'] . "'" . ((bool) $configs['titles'] ? ",
	                animationStart: function(){
	                    $('.caption').animate({
	                        bottom: -36
	                    },100);
	                },
	                animationComplete: function(current){
	                    $('.caption').animate({
	                        bottom: 0
	                    },200);
	                }" : "") . "
	            });
	        });"
        );
        
        $this->assign('configs', $configs);
    }

    public static function getStandardSettingValueList()
    {
        return array(
        	self::SETTING_WRAP_IN_BOX => false,
        	self::SETTING_SHOW_TITLE => false
        );
    }

    public static function getAccess()
    {
        return self::ACCESS_ALL;
    }
}