<?php

/**
 * This software is intended for use with Oxwall Free Community Software http://www.oxwall.org/ and is
 * licensed under The BSD license.

 * ---
 * Copyright (c) 2009, Skalfa LLC
 * All rights reserved.

 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:
 *
 *  - Redistributions of source code must retain the above copyright notice, this list of conditions and
 *  the following disclaimer.
 *
 *  - Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *  the following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *  - Neither the name of the Skalfa LLC nor the names of its contributors may be used to endorse or promote products
 *  derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * Image slider administration action controller
 *
 * @author Egor Bulgakov <plugins@oxwallcs.com>
 * @package ow.plugin.ocsslider.controllers
 * @since 1.0
 */
class OCSSLIDER_CTRL_Admin extends ADMIN_CTRL_Abstract
{
    /**
     * Default action
     */
    public function index()
    {
        $language = OW::getLanguage();

        $item = new BASE_MenuItem();
        $item->setLabel($language->text('ocsslider', 'admin_menu_images'));
        $item->setUrl(OW::getRouter()->urlForRoute('ocsslider.admin_sliders'));
        $item->setKey('sliders');
        $item->setIconClass('ow_ic_picture');
        $item->setOrder(0);
        
        $item2 = new BASE_MenuItem();
        $item2->setLabel($language->text('ocsslider', 'admin_menu_settings'));
        $item2->setUrl(OW::getRouter()->urlForRoute('ocsslider.admin_settings'));
        $item2->setKey('settings');
        $item2->setIconClass('ow_ic_gear_wheel');
        $item2->setOrder(1);

        $menu = new BASE_CMP_ContentMenu(array($item, $item2));
        $this->addComponent('menu', $menu);

        $this->setPageHeading(OW::getLanguage()->text('ocsslider', 'page_heading_admin_sliders'));
        $this->setPageHeadingIconClass('ow_ic_picture');
        
        $elem = $menu->getElement('sliders');
        if ( $elem )
        {
        	$elem->setActive(true);
        }
        
        $imageService = OCSSLIDER_BOL_ImageService::getInstance();
        
        $form = new AddImageForm();
        $this->addForm($form);
        
        $form2 = new EditImageForm();
        $this->addForm($form2);
        
        if ( OW::getRequest()->isPost() )
        {
        	switch ( $_POST['form_name'] )
        	{
        		case 'add-image-form':
        			if ( $form->isValid($_POST) )
        			{
        				$data = $form->getValues();
        				
	        			if ( !empty($_FILES['image']['tmp_name']) )
			            {
			                $ext = UTIL_File::getExtension($_FILES['image']['name']);
			                if ( !UTIL_File::validateImage($_FILES['image']['name']) )
			                {
			                    OW::getFeedback()->warning($language->text('ocsslider', 'not_image_file'));
			                    $this->redirect();
			                }
			                
			                $image = new OCSSLIDER_BOL_Image();
			                $image->hash = time();
			                $image->ext = $ext;
			                $image->order = $imageService->getNextOrder();
			                if ( !empty($data['href']) )
			                {
			                    $image->href = trim($data['href']);
			                }
			                if ( !empty($data['title']) )
			                {
			                    $image->title = trim($data['title']);
			                }
			                
			                if ( $imageService->addImage($image, $_FILES['image']) )
			                {
			                    OW::getFeedback()->info($language->text('ocsslider', 'image_added'));
			                }
			            }
        			} 
        			break;
        			
        		case 'edit-image-form':
                    if ( $form2->isValid($_POST) )
                    {
                        $data = $form2->getValues();
                        
                        if ( !$data['image_id'] || !$image = $imageService->findById($data['image_id']) )
                        {
                        	$this->redirect();
                        }
                        
                        if ( !empty($data['image_title']) )
                        {
                        	$image->title = trim($data['image_title']);
                        }
                        
                        if ( !empty($data['href']) )
                        {
                        	$image->href = trim($data['href']);
                        }
                        
                        $imageService->updateImageDto($image);
                        
                        if ( !empty($_FILES['image']['tmp_name']) )
                        {
                            $ext = UTIL_File::getExtension($_FILES['image']['name']);
                            if ( !UTIL_File::validateImage($_FILES['image']['name']) )
                            {
                                OW::getFeedback()->warning($language->text('ocsslider', 'not_image_file'));
                                $this->redirect();
                            }
                            
                            $image->ext = $ext;
                            $imageService->updateImageFile($image, $_FILES['image']['tmp_name']);
                        }
                        
                        OW::getFeedback()->info("Image info updated");
                    }
        			break;
        	}
        	
        	$this->redirect();
        }
        
        $images = $imageService->getImageList();
        $this->assign('images', $images);
        
        $this->assign('ajaxDeleteUrl', json_encode(OW::getRouter()->urlFor('OCSSLIDER_CTRL_Admin', 'ajaxDeleteImage')));
        $this->assign('ajaxReorderUrl', json_encode(OW::getRouter()->urlFor('OCSSLIDER_CTRL_Admin', 'ajaxReorderImage')));
        $language->addKeyForJs('ocsslider', 'confirm_image_delete');
    }
    
    public function ajaxDeleteImage()
    {
    	if ( empty($_POST['imageId']) )
    	{
    		echo json_encode(array('result' => false));
    		exit;
    	}
    	
    	if ( OCSSLIDER_BOL_ImageService::getInstance()->deleteImage((int)$_POST['imageId']) )
    	{
    		echo json_encode(array(
                'result' => true, 
                'message' => OW::getLanguage()->text('ocsslider', 'image_deleted'))
    		);
    	}
    	else
    	{
    		echo json_encode(array('result' => false));
    	}
    	
    	exit;
    }

    public function ajaxReorderImage()
    {
        if ( empty($_POST['imageId']) || empty($_POST['sort']) )
        {
            echo json_encode(array('result' => false));
            exit;
        }
        
        if ( OCSSLIDER_BOL_ImageService::getInstance()->changeImageOrder((int)$_POST['imageId'], $_POST['sort']) )
        {
            echo json_encode(array(
                'result' => true, 
                'message' => OW::getLanguage()->text('ocsslider', 'order_changed'))
            );
        }
        else
        {
            echo json_encode(array('result' => false));
        }
        
        exit;
    }
    
    public function ajaxGetImage( )
    {
        if ( !empty($_POST['id']) )
        {
            $image = OCSSLIDER_BOL_ImageService::getInstance()->findById($_POST['id']);
            
            if ( $image )
            {
                $return = array('href' => $image->href, 'title' => $image->title);
            }
        }
        else 
        {
            $return = array();
        }
        
        echo json_encode($return);
        
        exit;
    }
    
    public function settings()
    {
        $language = OW::getLanguage();

        $item = new BASE_MenuItem();
        $item->setLabel($language->text('ocsslider', 'admin_menu_images'));
        $item->setUrl(OW::getRouter()->urlForRoute('ocsslider.admin_sliders'));
        $item->setKey('sliders');
        $item->setIconClass('ow_ic_picture');
        $item->setOrder(0);
        
        $item2 = new BASE_MenuItem();
        $item2->setLabel($language->text('ocsslider', 'admin_menu_settings'));
        $item2->setUrl(OW::getRouter()->urlForRoute('ocsslider.admin_settings'));
        $item2->setKey('settings');
        $item2->setIconClass('ow_ic_gear_wheel');
        $item2->setOrder(1);

        $menu = new BASE_CMP_ContentMenu(array($item, $item2));
        $this->addComponent('menu', $menu);

        $this->setPageHeading(OW::getLanguage()->text('ocsslider', 'page_heading_admin_sliders'));
        $this->setPageHeadingIconClass('ow_ic_picture');
        
        $elem = $menu->getElement('settings');
        if ( $elem )
        {
            $elem->setActive(true);
        }

        $configs = OW::getConfig()->getValues('ocsslider');

        $form = new UpdateConfigForm();

        $this->addForm($form);

        if ( OW::getRequest()->isPost() && $form->isValid($_POST) )
        {
            $res = $form->process();
            OW::getFeedback()->info($language->text('ocsslider', 'settings_updated'));
            $this->redirect();
        }
        
        $configs = OW::getConfig()->getValue('ocsslider', 'configuration');
        $configs = json_decode($configs, true);
        
        foreach ( $configs as $key => $val )
        {
            $form->getElement($key)->setValue($val);	
        }
    }
}

class AddImageForm extends Form 
{
	public function __construct()
	{
		parent::__construct('add-image-form');

		$this->setEnctype(Form::ENCTYPE_MULTYPART_FORMDATA);
		
		$lang = OW::getLanguage();
		
        $fileField = new FileField('image');
        $fileField->setLabel($lang->text('ocsslider', 'image'));
        $this->addElement($fileField);
        
        $hrefField = new TextField('href');
        $hrefField->setLabel($lang->text('ocsslider', 'href'));
        $this->addElement($hrefField);
        
        $title = new TextField('title');
        $title->setLabel($lang->text('ocsslider', 'title'));
        $this->addElement($title);
        
        $submit = new Submit('add');
        $this->addElement($submit);
	}
}

class EditImageForm extends Form 
{
    public function __construct()
    {
        parent::__construct('edit-image-form');

        $this->setEnctype(Form::ENCTYPE_MULTYPART_FORMDATA);
        
        $lang = OW::getLanguage();
        
        $idField = new HiddenField('image_id');
        $this->addElement($idField);
        
        $fileField = new FileField('image');
        $fileField->setLabel($lang->text('ocsslider', 'image'));
        $this->addElement($fileField);
        
        $hrefField = new TextField('href');
        $hrefField->setLabel($lang->text('ocsslider', 'href'));
        $this->addElement($hrefField);
        
        $title = new TextField('image_title');
        $title->setLabel($lang->text('ocsslider', 'title'));
        $this->addElement($title);
        
        $submit = new Submit('update');
        $this->addElement($submit);
    }
}

class UpdateConfigForm extends Form
{
    /**
     * Class constructor
     */
    public function __construct()
    {
        parent::__construct('update-config-form');

        $lang = OW::getLanguage();

        $width = new TextField('width');
        $width->setRequired(true);
        $width->setLabel($lang->text('ocsslider', 'width'));
        $this->addElement($width);
        
        $height = new TextField('height');
        $height->setRequired(true);
        $height->setLabel($lang->text('ocsslider', 'height'));
        $this->addElement($height);
        
        $titles = new CheckboxField('titles');
        $titles->setLabel($lang->text('ocsslider', 'titles'));
        $this->addElement($titles);
        
        $effect = new Selectbox('effect');
        $effect->setRequired(true);
        $options = array(
            'slide' => $lang->text('ocsslider', 'effect_slide'),
            'fade' => $lang->text('ocsslider', 'effect_fade')
        );
        $effect->setOptions($options);
        $effect->setLabel($lang->text('ocsslider', 'effect'));
        $this->addElement($effect);
        
        $pagination = new CheckboxField('pagination');
        $pagination->setLabel($lang->text('ocsslider', 'pagination'));
        $this->addElement($pagination);
        
        $preload = new CheckboxField('preload');
        $preload->setLabel($lang->text('ocsslider', 'preload'));
        $this->addElement($preload);
        
        $navigation = new CheckboxField('navigation');
        $navigation->setLabel($lang->text('ocsslider', 'navigation'));
        $this->addElement($navigation);
        
        $speed = new TextField('speed');
        $speed->setLabel($lang->text('ocsslider', 'speed'));
        $this->addElement($speed);
        
        $play = new TextField('play');
        $play->setLabel($lang->text('ocsslider', 'play'));
        $this->addElement($play);
        
        $crossfade = new CheckboxField('crossfade');
        $crossfade->setLabel($lang->text('ocsslider', 'crossfade'));
        $this->addElement($crossfade);
        
        $randomize = new CheckboxField('randomize');
        $randomize->setLabel($lang->text('ocsslider', 'randomize'));
        $this->addElement($randomize);
        
        $hoverPause = new CheckboxField('hoverPause');
        $hoverPause->setLabel($lang->text('ocsslider', 'hover_pause'));
        $this->addElement($hoverPause);
        
        // submit
        $submit = new Submit('save');
        $submit->setValue($lang->text('admin', 'save_btn_label'));
        $this->addElement($submit);
    }

    /**
     * Updates photo plugin configuration
     *
     * @return boolean
     */
    public function process()
    {
        $values = $this->getValues();

	    $confArr = array('width','height','effect','pagination','preload','navigation','speed','play','crossfade','randomize','hoverPause','titles');

	    $config = OW::getConfig();
	    $setup = array();
        foreach ( $values as $key => $val )
        {
        	if ( in_array($key, $confArr) )
        	{
        		$setup[$key] = $val;
        	}
        }
        
        $config->saveConfig('ocsslider', 'configuration', json_encode($setup));

        return array('result' => true);
    }
}